/** We have 3 CPUs in use. 
1st CPU is placed outside the house on the door.
2nd CPU is used to detected someone at the door (after unnecessary few steps, like domino falling).
3rd CPU is placed inside the house with the house owner. 
In our explanation we will refer to them as 1st, 2nd and 3rd CPUs. 

Steps:
1.  Someone pulls a string by the door. The 2nd CPU senses light change and sends a message to 1st CPU using MQTT (through 
    adafruit.io) that someone is at the door. 
    How is the message sent? 
    2nd CPU publishes a light value to a light feed which 1st CPU is subscribed to.
2.  1st CPU changes the value of virtual pin VO to 2 on Blynk. Using a Blynk webhook, the change of V0 value to 2 triggers a 
    make.com scenario.
3.  A line is added to google sheets with time of the new visits and adds 1 to a counter counting all visits. 
4.  1st CPU lets 3rd CPU know someone is at the door using Blynk and make.com. 
    How? 
    The new counter value is retrieved from google sheets, then it is send using an HTTP post request to the address of 
    the 3rd CPU webhook.
5.  Using Make.com a random happy gif is added to a Facebook post on the house owner’s Facebook page, posting
    “YAY! I have a new visitor” with the chosen gif. 
6.  The HTTP request (step 4) using Blynk webhook, triggers an alarm noise on 3rd CPU. The noise alerts the house owner 
    someone is at the door.
7.  On the house owner blink’s app it says “Wow, u have had X visitors this month!” and average rate of accepted entrances
8.  When it is 7pm virtual pin VO is changed to 1 (1st CPU code). Then, using make.com scenario and router we ask 
    chat gpt to come up with a funny sentence in Spanish to cheer up the person. 
    Then using a TextCortexAI translator (free version of google translate), the sentence is translated to English.
9.  The sentence is sent to the house owner’s email account through gmail (using Make.com), saying 
    “Hey Buddy, I know it has been a long day but I just wanted to say: {the translated sentence} :)”. 
10. 3rd CPU senses temperature in the house and sends it to the 1st CPU. How? 
3rd CPU sets virtual pin V2 value to the detected temperature. The 1st CPU is updated on V2 value using a webhook 
    (V2 callback function is on the 1st CPU code).
11. If temp is below 16 Celsius degrees, the house owner’s blynk app says “Don't forget a jacket!”, 
    if it’s between 16-30 it says “Weather is wonderful inside!” and if it’s above 30 it says “It's boiling inside. T shirts only!”. 
12. If house owner presses right button on 3rd CPU, 1st CPU shows green lights and says “please go in”. 
    Else, house owner presses left button on 3rd CPU 1st CPU shows red lights and says “sorry you can not go in”. 
    how does it work? 
    By changing value of virtual pin V1 on 3rd CPU code, the new value is updated on 3rd CPU (V1 callback function is on 1st CPU code). 
    If V1==1 entrance allowed, if V1==0 entrance denied. 

This code is the 3rd CPU code.

  The circuit:
    No components attached to input/output.
    Sensors used- light.
    Uses Blynk, make.com, adafruit.io (mqtt).

  Video link: https://drive.google.com/file/d/1NGyTcrLBZ12IYkXyvyJryrFrds027Fxj/view?usp=sharing
  Created By: 
    Shira Cohen ID 318971827
    Nadav Poran ID 206933954
    Kylie Koshet ID 206862708

**/

// Importing necesary libraries
#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
#include <TimeLib.h>

// setting up blynk 
#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB
#define BLYNK_TEMPLATE_ID "TMPL67Ikve723"
#define BLYNK_TEMPLATE_NAME "FinalProject"
#define BLYNK_AUTH_TOKEN "UZFhs2gdu9euOl5mpJQGJSQHNos8gP5f"

#define EspSerial Serial1
#define ESP8266_BAUD 115200

char auth[] = BLYNK_AUTH_TOKEN;
char ssid[] = "wireless";
char pass[] = "5101051010";


ESP8266 wifi(&EspSerial);

long last_sent = 0;
int temp = 0;

void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(115200);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
}

int newVisitor = 0;
boolean allowClick = true;
unsigned long timer = millis();
boolean playedSound = false;
double visitorsToday = 0;
double visitorsAllowedToday = 0;

void loop() {
  Blynk.run();

  // number of visitors is shown on the house owners blynk app
  if(newVisitor > 0){
    Blynk.virtualWrite(V0, newVisitor + 1);
    newVisitor = 0;
    allowClick = false;
    lightUpLEDs(0,0,255);
    if(!playedSound){
      playSound();
    }
  }

  if (CircuitPlayground.rightButton() && !allowClick) {
    Serial.println("2");
    allowClick = true;
    visitorsAllowedToday++;
    visitorsToday++;
    Blynk.virtualWrite(V2, "Your entrance ratio today is " + String(visitorsAllowedToday / visitorsToday));
    temp = CircuitPlayground.temperature();
    lightUpLEDs(0,0,0);
    playedSound = false;
    Blynk.virtualWrite(V1, "1:" + String(temp));
  }

  if (CircuitPlayground.leftButton() && !allowClick) {
    Serial.println("3");
    allowClick = true;
    visitorsToday++;
    Blynk.virtualWrite(V2, "Your entrance ratio today is " + String(visitorsAllowedToday / visitorsToday));
    temp = CircuitPlayground.temperature();
    lightUpLEDs(0,0,0);
    playedSound = false;
    Blynk.virtualWrite(V1, "0:" + String(temp));
  }

  time_t timeNow = now();
  int currentHour = hour(timeNow);
  int currentMinute = minute(timeNow);

  // if its 12AM reset
  if (currentHour == 0 && currentMinute == 0) {
    visitorsToday = 0;
    visitorsAllowedToday = 0;
  }
}

BLYNK_WRITE(V0) {
  newVisitor = param.asInt();
}


void lightUpLEDs(int redValue, int greenValue, int blueValue) {
  for (int i = 0; i < 10; i++) {
    CircuitPlayground.setPixelColor(i, redValue, greenValue, blueValue);
  } 
}

void playSound(){
  for (int i = 440; i < 1760; i += 100) {
    CircuitPlayground.playTone(i, 100);
  }  
  playedSound = true;
}
